import { DOMWindow } from "jsdom";
import { IHandlerOutput } from "./handler.interface";
import { EngineParseError } from "../errors/main";

export default async function google(
  window: DOMWindow,
): Promise<IHandlerOutput> {
  const googleAnchors = [
    ...window.document.querySelectorAll("a[jsname=ACyKwe]"),
  ] as HTMLAnchorElement[];

  const results = googleAnchors
    .map((a: HTMLAnchorElement): GoogleProps => {
      const parsedHref = new URL(new URL(a.href).searchParams.get("url")!);
      return {
        href: a.href!,
        siteName: parsedHref.hostname,
        heading: a.childNodes[1]?.textContent,
      };
    })
    .filter((a) => a.heading);

  if (!googleAnchors) {
    throw new EngineParseError(
      "Failed to find anchors in search result [google]",
    );
  }

  const convertToFormat = (result: GoogleProps, isHtml: boolean) => {
    return isHtml
      ? `<p><a href="${result.href}">${result.siteName} - ${result.heading}</p>`
      : `${result.siteName} - ${result.heading} > ${result.href}`;
  };

  const content = results.map((result) => {
    return convertToFormat(result, true);
  });

  const textContent = results.map((result) => {
    return convertToFormat(result, false);
  });

  const search = window.document.getElementById(
    "APjFqb",
  ) as HTMLTextAreaElement;

  const navLinks = [
    ...window.document.querySelectorAll(
      "table[class=AaVjTc] > tbody > tr > td > a",
    ),
  ].map((l) => {
    const link = l as HTMLAnchorElement;
    return `<td><a href="${link.href}">${link.innerHTML}</a></td>`;
  });

  const currPage =
    (window.document.querySelector(".YyVfkd") as HTMLTableCellElement)
      ?.cellIndex || 0;

  const pageTd = `<td>${currPage}</td>`;

  if (currPage === 1) navLinks.splice(currPage - 1, 0, pageTd);
  else if (currPage !== 0) navLinks.splice(currPage, 0, pageTd);

  const navigation = `<table>
    <tbody><tr>${navLinks.join("")}</tr></tbody>
  </table>`;

  const searchForm = `
  <form onsubmit="window.location.href = '/get?url=https://www.google.com/search?q='+ document.getElementById('q').value.split(' ').join('+'); return false">
    <input type="text" name="q" id="q" value="${search?.value}">
    <input type="button" value="Search" onclick="window.location.href = '/get?url=https://www.google.com/search?q='+ document.getElementById('q').value.split(' ').join('+');">
  </form>
  `;

  return {
    content: `${searchForm}${content.join("")}${navigation}`,
    textContent: textContent.join("\n"),
    title: window.document.title,
    lang: window.document.documentElement.lang,
  };
}

export const GoogleDomains = [
  "google.com",
  "google.ad",
  "google.ae",
  "google.com.af",
  "google.com.ag",
  "google.al",
  "google.am",
  "google.co.ao",
  "google.com.ar",
  "google.as",
  "google.at",
  "google.com.au",
  "google.az",
  "google.ba",
  "google.com.bd",
  "google.be",
  "google.bf",
  "google.bg",
  "google.com.bh",
  "google.bi",
  "google.bj",
  "google.com.bn",
  "google.com.bo",
  "google.com.br",
  "google.bs",
  "google.bt",
  "google.co.bw",
  "google.by",
  "google.com.bz",
  "google.ca",
  "google.cd",
  "google.cf",
  "google.cg",
  "google.ch",
  "google.ci",
  "google.co.ck",
  "google.cl",
  "google.cm",
  "google.cn",
  "google.com.co",
  "google.co.cr",
  "google.com.cu",
  "google.cv",
  "google.com.cy",
  "google.cz",
  "google.de",
  "google.dj",
  "google.dk",
  "google.dm",
  "google.com.do",
  "google.dz",
  "google.com.ec",
  "google.ee",
  "google.com.eg",
  "google.es",
  "google.com.et",
  "google.fi",
  "google.com.fj",
  "google.fm",
  "google.fr",
  "google.ga",
  "google.ge",
  "google.gg",
  "google.com.gh",
  "google.com.gi",
  "google.gl",
  "google.gm",
  "google.gr",
  "google.com.gt",
  "google.gy",
  "google.com.hk",
  "google.hn",
  "google.hr",
  "google.ht",
  "google.hu",
  "google.co.id",
  "google.ie",
  "google.co.il",
  "google.im",
  "google.co.in",
  "google.iq",
  "google.is",
  "google.it",
  "google.je",
  "google.com.jm",
  "google.jo",
  "google.co.jp",
  "google.co.ke",
  "google.com.kh",
  "google.ki",
  "google.kg",
  "google.co.kr",
  "google.com.kw",
  "google.kz",
  "google.la",
  "google.com.lb",
  "google.li",
  "google.lk",
  "google.co.ls",
  "google.lt",
  "google.lu",
  "google.lv",
  "google.com.ly",
  "google.co.ma",
  "google.md",
  "google.me",
  "google.mg",
  "google.mk",
  "google.ml",
  "google.com.mm",
  "google.mn",
  "google.com.mt",
  "google.mu",
  "google.mv",
  "google.mw",
  "google.com.mx",
  "google.com.my",
  "google.co.mz",
  "google.com.na",
  "google.com.ng",
  "google.com.ni",
  "google.ne",
  "google.nl",
  "google.no",
  "google.com.np",
  "google.nr",
  "google.nu",
  "google.co.nz",
  "google.com.om",
  "google.com.pa",
  "google.com.pe",
  "google.com.pg",
  "google.com.ph",
  "google.com.pk",
  "google.pl",
  "google.pn",
  "google.com.pr",
  "google.ps",
  "google.pt",
  "google.com.py",
  "google.com.qa",
  "google.ro",
  "google.ru",
  "google.rw",
  "google.com.sa",
  "google.com.sb",
  "google.sc",
  "google.se",
  "google.com.sg",
  "google.sh",
  "google.si",
  "google.sk",
  "google.com.sl",
  "google.sn",
  "google.so",
  "google.sm",
  "google.sr",
  "google.st",
  "google.com.sv",
  "google.td",
  "google.tg",
  "google.co.th",
  "google.com.tj",
  "google.tl",
  "google.tm",
  "google.tn",
  "google.to",
  "google.com.tr",
  "google.tt",
  "google.com.tw",
  "google.co.tz",
  "google.com.ua",
  "google.co.ug",
  "google.co.uk",
  "google.com.uy",
  "google.co.uz",
  "google.com.vc",
  "google.co.ve",
  "google.co.vi",
  "google.com.vn",
  "google.vu",
  "google.ws",
  "google.rs",
  "google.co.za",
  "google.co.zm",
  "google.co.zw",
  "google.cat",
  "www.google.com",
  "www.google.ad",
  "www.google.ae",
  "www.google.com.af",
  "www.google.com.ag",
  "www.google.al",
  "www.google.am",
  "www.google.co.ao",
  "www.google.com.ar",
  "www.google.as",
  "www.google.at",
  "www.google.com.au",
  "www.google.az",
  "www.google.ba",
  "www.google.com.bd",
  "www.google.be",
  "www.google.bf",
  "www.google.bg",
  "www.google.com.bh",
  "www.google.bi",
  "www.google.bj",
  "www.google.com.bn",
  "www.google.com.bo",
  "www.google.com.br",
  "www.google.bs",
  "www.google.bt",
  "www.google.co.bw",
  "www.google.by",
  "www.google.com.bz",
  "www.google.ca",
  "www.google.cd",
  "www.google.cf",
  "www.google.cg",
  "www.google.ch",
  "www.google.ci",
  "www.google.co.ck",
  "www.google.cl",
  "www.google.cm",
  "www.google.cn",
  "www.google.com.co",
  "www.google.co.cr",
  "www.google.com.cu",
  "www.google.cv",
  "www.google.com.cy",
  "www.google.cz",
  "www.google.de",
  "www.google.dj",
  "www.google.dk",
  "www.google.dm",
  "www.google.com.do",
  "www.google.dz",
  "www.google.com.ec",
  "www.google.ee",
  "www.google.com.eg",
  "www.google.es",
  "www.google.com.et",
  "www.google.fi",
  "www.google.com.fj",
  "www.google.fm",
  "www.google.fr",
  "www.google.ga",
  "www.google.ge",
  "www.google.gg",
  "www.google.com.gh",
  "www.google.com.gi",
  "www.google.gl",
  "www.google.gm",
  "www.google.gr",
  "www.google.com.gt",
  "www.google.gy",
  "www.google.com.hk",
  "www.google.hn",
  "www.google.hr",
  "www.google.ht",
  "www.google.hu",
  "www.google.co.id",
  "www.google.ie",
  "www.google.co.il",
  "www.google.im",
  "www.google.co.in",
  "www.google.iq",
  "www.google.is",
  "www.google.it",
  "www.google.je",
  "www.google.com.jm",
  "www.google.jo",
  "www.google.co.jp",
  "www.google.co.ke",
  "www.google.com.kh",
  "www.google.ki",
  "www.google.kg",
  "www.google.co.kr",
  "www.google.com.kw",
  "www.google.kz",
  "www.google.la",
  "www.google.com.lb",
  "www.google.li",
  "www.google.lk",
  "www.google.co.ls",
  "www.google.lt",
  "www.google.lu",
  "www.google.lv",
  "www.google.com.ly",
  "www.google.co.ma",
  "www.google.md",
  "www.google.me",
  "www.google.mg",
  "www.google.mk",
  "www.google.ml",
  "www.google.com.mm",
  "www.google.mn",
  "www.google.com.mt",
  "www.google.mu",
  "www.google.mv",
  "www.google.mw",
  "www.google.com.mx",
  "www.google.com.my",
  "www.google.co.mz",
  "www.google.com.na",
  "www.google.com.ng",
  "www.google.com.ni",
  "www.google.ne",
  "www.google.nl",
  "www.google.no",
  "www.google.com.np",
  "www.google.nr",
  "www.google.nu",
  "www.google.co.nz",
  "www.google.com.om",
  "www.google.com.pa",
  "www.google.com.pe",
  "www.google.com.pg",
  "www.google.com.ph",
  "www.google.com.pk",
  "www.google.pl",
  "www.google.pn",
  "www.google.com.pr",
  "www.google.ps",
  "www.google.pt",
  "www.google.com.py",
  "www.google.com.qa",
  "www.google.ro",
  "www.google.ru",
  "www.google.rw",
  "www.google.com.sa",
  "www.google.com.sb",
  "www.google.sc",
  "www.google.se",
  "www.google.com.sg",
  "www.google.sh",
  "www.google.si",
  "www.google.sk",
  "www.google.com.sl",
  "www.google.sn",
  "www.google.so",
  "www.google.sm",
  "www.google.sr",
  "www.google.st",
  "www.google.com.sv",
  "www.google.td",
  "www.google.tg",
  "www.google.co.th",
  "www.google.com.tj",
  "www.google.tl",
  "www.google.tm",
  "www.google.tn",
  "www.google.to",
  "www.google.com.tr",
  "www.google.tt",
  "www.google.com.tw",
  "www.google.co.tz",
  "www.google.com.ua",
  "www.google.co.ug",
  "www.google.co.uk",
  "www.google.com.uy",
  "www.google.co.uz",
  "www.google.com.vc",
  "www.google.co.ve",
  "www.google.co.vi",
  "www.google.com.vn",
  "www.google.vu",
  "www.google.ws",
  "www.google.rs",
  "www.google.co.za",
  "www.google.co.zm",
  "www.google.co.zw",
  "www.google.cat",
];

interface GoogleProps {
  href: string;
  siteName: string;
  heading: string | null;
}
